@@ -1,6 +1,7 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_init.c - Redland library initialisation / termination
+ * rdf_init.c - Redland library initialisation / termination and memory
+ *              management
  *
  * Copyright (C) 2000-2011, David Beckett http://www.dajobe.org/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
@@ -142,7 +143,7 @@ librdf_new_world(void)
   struct timezone tz;
 #endif
 
-  world = (librdf_world*)LIBRDF_CALLOC(librdf_world, sizeof(*world), 1);
+  world = LIBRDF_CALLOC(librdf_world*, 1, sizeof(*world));
 
   if(!world)
     return NULL;
@@ -161,8 +162,10 @@ librdf_new_world(void)
   world->ltdl_opened = !(lt_dlinit());
   if (world->ltdl_opened)
     lt_dlsetsearchpath(REDLAND_MODULE_PATH);
+#ifdef LIBRDF_DEBUG
   else
     LIBRDF_DEBUG1("lt_dlinit() failed\n");
+#endif
 #endif
 
   return world;
@@ -195,12 +198,13 @@ librdf_free_world(librdf_world *world)
   librdf_finish_node(world);
   librdf_finish_uri(world);
 
+  /* Uses rdf_hash so free before destroying hashes */
+  librdf_finish_raptor(world);
+
   librdf_finish_hash(world);
 
   librdf_finish_digest(world);
 
-  librdf_finish_raptor(world);
-
 #ifdef WITH_THREADS
 
   if(world->hash_datums_mutex) {
@@ -282,15 +286,15 @@ librdf_world_open(librdf_world *world)
   
   librdf_world_init_mutex(world);
 
-  /* Initialize raptor library first. Used by many other classes. */
-  librdf_init_raptor(world);
-  
   /* Digests second, lots of things use these */
   librdf_init_digest(world);
 
   /* Hash next, needed for URIs */
   librdf_init_hash(world);
 
+  /* Initialize raptor; uses rdf_hash for bnode mapping */
+  librdf_init_raptor(world);
+  
   librdf_init_uri(world);
   librdf_init_node(world);
 
@@ -485,7 +489,7 @@ librdf_world_set_feature(librdf_world* world, librdf_uri* feature,
 #ifdef WITH_THREADS
       pthread_mutex_lock(world->mutex);
 #endif
-      world->genid_base = 1;
+      world->genid_base = i;
 #ifdef WITH_THREADS
       pthread_mutex_unlock(world->mutex);
 #endif
@@ -502,7 +506,7 @@ librdf_world_set_feature(librdf_world* world, librdf_uri* feature,
 #ifdef WITH_THREADS
       pthread_mutex_lock(world->mutex);
 #endif
-      world->genid_counter = 1;
+      world->genid_counter = i;
 #ifdef WITH_THREADS
       pthread_mutex_unlock(world->mutex);
 #endif
@@ -555,7 +559,7 @@ librdf_world_get_genid(librdf_world* world)
   while(tmppid /= 10)
     length++;
   
-  buffer = (unsigned char*)LIBRDF_MALLOC(cstring, length);
+  buffer = LIBRDF_MALLOC(unsigned char*, length);
   if(!buffer)
     return NULL;
 
@@ -646,6 +650,58 @@ librdf_basename(const char *name)
 }
 
 
+/**
+ * librdf_free_memory:
+ * @ptr: pointer to free
+ *
+ * Free memory allocated in the library.
+ *
+ * Required for some runtimes where memory must be freed within the same
+ * shared object it was allocated in.
+ **/
+void
+librdf_free_memory(void *ptr)
+{
+  raptor_free_memory(ptr);
+}
+
+
+/**
+ * librdf_alloc_memory:
+ * @size: alloc size
+ *
+ * Allocate memory inside the library similar to malloc().
+ *
+ * Required for some runtimes where memory must be freed within the same
+ * shared object it was allocated in.
+ *
+ * Return value: pointer to memory or NULL on failure
+ **/
+void*
+librdf_alloc_memory(size_t size)
+{
+  return raptor_alloc_memory(size);
+}
+
+/**
+ * librdf_calloc_memory:
+ * @nmemb: number of members
+ * @size: size of member
+ *
+ * Allocate zeroed array of items inside the library similar to calloc().
+ *
+ * Required for some runtimes where memory must be freed within the same
+ * shared object it was allocated in.
+ * 
+ * Return value: pointer to memory or NULL on failure
+ **/ 
+void*
+librdf_calloc_memory(size_t nmemb, size_t size)
+{
+  return raptor_calloc_memory(nmemb, size);
+}
+
+
 #if defined (LIBRDF_DEBUG) && defined(LIBRDF_MEMORY_SIGN)
 void*
 librdf_sign_malloc(size_t size)
@@ -803,7 +859,7 @@ main(int argc, char *argv[])
     return 1;
   }
   fprintf(stdout, "%s: New identifier is: '%s'\n", program, id);
-  LIBRDF_FREE(cstring, id);
+  LIBRDF_FREE(char*, id);
 
   fprintf(stdout, "%s: Deleting world\n", program);
   librdf_free_world(world);
