@@ -28,7 +28,7 @@
 #endif
 
 #ifdef WIN32
-#include <win32_config.h>
+#include <win32_rdf_config.h>
 #endif
 
 #include <stdio.h>
@@ -43,7 +43,9 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include <redland.h>
 #include <raptor.h>
@@ -164,7 +166,7 @@ static const char *default_storage_name="hashes";
 static const char *default_storage_options="hash-type='bdb',dir='.'";
 
 
-static int
+static int REDLAND_CALLBACK_STDCALL
 log_handler(void *user_data, librdf_log_message *message) 
 {
   /* int code=message->code; */ /* The error code */
@@ -1556,7 +1558,7 @@ main(int argc, char *argv[])
     librdf_free_query(query);
 
   if(free_uri_string)
-    free(uri_string);
+    raptor_free_memory(uri_string);
 
   if(output_model) {
     if(librdf_serializer_serialize_model_to_file_handle(output_serializer, 
